//----------------------------------------------------------------------------------
// File:   PerlinFire.cpp
// Author: Andrew Tatarinov
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#include "DXUT.h"
#include "DXUTgui.h"
#include "DXUTmisc.h"
#include "DXUTShapes.h"
#include "DXUTCamera.h"
#include "DXUTSettingsDlg.h"
#include "SDKmisc.h"
#include "SDKmesh_old.h"

#define MIN_SAMPLING_RATE   4
#define MAX_SAMPLING_RATE   64

CModelViewerCamera g_Camera;
CDXUTDialogResourceManager g_DialogResourceManager; // manager for shared resources of dialogs
CD3DSettingsDlg g_SettingsDlg; // Device settings dialog
CDXUTDialog g_HUD; // dialog for standard controls
CDXUTDialog g_SampleUI; // dialog for sample specific controls

// Direct3D 10 resources
ID3DX10Font * g_pFont = NULL;
ID3DX10Sprite * g_pSprite = NULL;
ID3D10Effect * g_pEffect = NULL;
ID3D10StateBlock * g_pCurrentState = NULL;  // stateblock for tracking state

// DX10 device and rendering related variables

ID3D10InputLayout * g_pGeometryVertexLayout   = NULL;

// DX10 meshes

CDXUTMesh10 g_OutsideWorldMesh;
ID3DX10Mesh * g_pBoxMesh = NULL;

// Shader variables and matrices

ID3D10EffectMatrixVariable * g_pmCubeViewMatrixVariable = NULL;
ID3D10EffectMatrixVariable * g_pmCubeProjMatrixVariable = NULL;
ID3D10EffectMatrixVariable * g_pmWorldViewProj = NULL;
ID3D10EffectVectorVariable * g_pvEyePos = NULL;
ID3D10EffectVectorVariable * g_pvLightPos = NULL;
ID3D10EffectScalarVariable * g_pfLightIntensity = NULL;
ID3D10EffectScalarVariable * g_pfStepSize = NULL;
ID3D10EffectScalarVariable * g_pfTime = NULL;
ID3D10EffectScalarVariable * g_pfNoiseScale = NULL;
ID3D10EffectScalarVariable * g_pfRoughness = NULL;
ID3D10EffectScalarVariable * g_pfFrequencyWeights = NULL;
ID3D10EffectScalarVariable * g_pbJitter = NULL;
ID3D10EffectScalarVariable * g_piCubeMapFace = NULL;

// Textures and related shader resource views

ID3D10Texture2D * g_pDepthBuffer = NULL;
ID3D10ShaderResourceView * g_pDepthBufferSRV = NULL;
ID3D10DepthStencilView * g_pDepthBufferDSV = NULL;

ID3D10Texture2D * g_pFireTexture = NULL;
ID3D10ShaderResourceView * g_pFireTextureSRV = NULL;

ID3D10Texture2D * g_pNoiseTexture = NULL;
ID3D10ShaderResourceView * g_pJitterTextureSRV = NULL;
ID3D10ShaderResourceView * g_pPermTextureSRV = NULL;

ID3D10EffectShaderResourceVariable * g_ptxScreenDepth = NULL;
ID3D10EffectShaderResourceVariable * g_ptxSceneTexture = NULL;
ID3D10EffectShaderResourceVariable * g_ptxFireTex = NULL;
ID3D10EffectShaderResourceVariable * g_ptxJitterTex = NULL;
ID3D10EffectShaderResourceVariable * g_ptxPermTex = NULL;

// Textures and views for shadow mapping

ID3D10Texture2D * g_pCubeMapDepth = NULL;
ID3D10DepthStencilView * g_pCubeMapDepthViewArray[6] = { NULL, NULL, NULL, NULL, NULL, NULL };
ID3D10ShaderResourceView * g_pCubeMapTextureRV = NULL;
ID3D10EffectShaderResourceVariable * g_pCubeMapTextureVariable = NULL;


// Effect techniques
ID3D10EffectTechnique * g_pCurrentTechnique = NULL;
ID3D10EffectTechnique * g_pPerlinFire3D = NULL;
ID3D10EffectTechnique * g_pPerlinFire4D = NULL;
ID3D10EffectTechnique * g_pPerlinFire3DFlow = NULL;
ID3D10EffectTechnique * g_pGeometryTechnique = NULL;
ID3D10EffectTechnique * g_pGeometryTechniqueAux = NULL;

// GUI related variables

CDXUTDialog g_SliderBar;
CDXUTTextHelper * g_pTextHelper   = NULL;

//--------------------------------------------------------------------------------------
// UI control IDs
//--------------------------------------------------------------------------------------
#define IDC_STATIC                  -1
#define IDC_TOGGLEFULLSCREEN        1
#define IDC_TOGGLEREF               2
#define IDC_CHANGEDEVICE            3
#define IDC_RESET                   4

#define IDC_USEJITTERING            5
#define IDC_FIRETECHNIQUE           6
#define IDC_SAMPLINGRATE_STATIC     7
#define IDC_SAMPLINGRATE            8

#define IDC_NOISESCALE_STATIC       9
#define IDC_NOISESCALE              10
#define IDC_ROUGHNESS_STATIC        11
#define IDC_ROUGHNESS               12
#define IDC_SHAPESIZE_STATIC        13
#define IDC_SHAPESIZE               14

#define IDC_FREQUENCIES_STATIC      15
#define IDC_FREQUENCY1              16
#define IDC_FREQUENCY2              17
#define IDC_FREQUENCY3              18
#define IDC_FREQUENCY4              19
#define IDC_FREQUENCY5              20

// Fire parameters

#define DEFAULT_JITTER true
#define DEFAULT_SAMPLING_RATE 16
#define DEFAULT_SPEED 0.6f
#define DEFAULT_NOISE_SCALE 1.35f
#define DEFAULT_ROUGHNESS 3.20f
#define DEFAULT_SHAPE_SIZE 3.0f
#define DEFAULT_FREQUENCY1 1.0f
#define DEFAULT_FREQUENCY2 0.5f
#define DEFAULT_FREQUENCY3 0.25f
#define DEFAULT_FREQUENCY4 0.125f
#define DEFAULT_FREQUENCY5 0.0625f

bool g_bJitter = DEFAULT_JITTER;
int g_nSamplingRate = DEFAULT_SAMPLING_RATE;
float g_fSpeed = DEFAULT_SPEED;
float g_fNoiseScale = DEFAULT_NOISE_SCALE;
float g_fRoughness = DEFAULT_ROUGHNESS;
float g_fShapeSize = DEFAULT_SHAPE_SIZE;
float g_fFrequencyWeights [5] = { DEFAULT_FREQUENCY1,
                                  DEFAULT_FREQUENCY2, 
                                  DEFAULT_FREQUENCY3, 
                                  DEFAULT_FREQUENCY4, 
                                  DEFAULT_FREQUENCY5 };
int g_CubeMapSize = 800;

bool LoadTexture2D(ID3D10Device * pd3dDevice, LPCWSTR fileName, ID3D10Texture2D ** tex, ID3D10ShaderResourceView ** texRV)
{
    HRESULT hr;
    ID3D10Resource * pRes = NULL;

    wchar_t str[MAX_PATH];
    if ( FAILED( DXUTFindDXSDKMediaFileCch( str, MAX_PATH, fileName ) ) ) {
        return false;
    }

    if ( FAILED( D3DX10CreateTextureFromFile( pd3dDevice, str, NULL, NULL, &pRes, &hr ) ) || (!pRes) ) {
        DXUTTRACE (L"Error while creating a texture from file %s\n", fileName);
        MessageBox (NULL, L"Failed to load texture", L"Error", MB_OK);
        return false;
    }

    V( pRes->QueryInterface( __uuidof(ID3D10Texture2D), (LPVOID*) tex) );

    D3D10_TEXTURE2D_DESC desc;
    (*tex)->GetDesc(& desc);

    D3D10_SHADER_RESOURCE_VIEW_DESC descSRV;
    ZeroMemory( &descSRV, sizeof(descSRV) );
    descSRV.Format = desc.Format;
    descSRV.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
    descSRV.Texture2D.MipLevels = desc.MipLevels;

    V( pd3dDevice->CreateShaderResourceView( *tex, &descSRV, texRV ) );

    pRes->Release();

    return true;
}

// Prepare cube map texture array

int PrepareCubeMap(ID3D10Device * pd3dDevice)
{
    // Create cubic depth stencil texture.
    
    D3D10_TEXTURE2D_DESC dstex;

    dstex.Width = g_CubeMapSize;
    dstex.Height = g_CubeMapSize;
    dstex.MipLevels = 1;
    dstex.ArraySize = 6;
    dstex.SampleDesc.Count = 1;
    dstex.SampleDesc.Quality = 0;
    dstex.Format = DXGI_FORMAT_R24G8_TYPELESS;
    dstex.Usage = D3D10_USAGE_DEFAULT;
    dstex.BindFlags = D3D10_BIND_DEPTH_STENCIL | D3D10_BIND_SHADER_RESOURCE;
    dstex.CPUAccessFlags = 0;
    dstex.MiscFlags = D3D10_RESOURCE_MISC_TEXTURECUBE;

    if( FAILED( pd3dDevice->CreateTexture2D( &dstex, NULL, &g_pCubeMapDepth ) ) )
    {
        DXUTTRACE( L"Failed to create depth stencil texture\n" );
        return false;
    }

    // Create the depth stencil view for the entire cube

    D3D10_DEPTH_STENCIL_VIEW_DESC DescDS;

    for( int i = 0; i < 6; i ++ )
    {
        DescDS.Format = DXGI_FORMAT_D24_UNORM_S8_UINT;
        DescDS.ViewDimension = D3D10_DSV_DIMENSION_TEXTURE2DARRAY;
        DescDS.Texture2DArray.FirstArraySlice = i;
        DescDS.Texture2DArray.ArraySize = (unsigned) 1;
        DescDS.Texture2DArray.MipSlice = 0;

        if( FAILED( pd3dDevice->CreateDepthStencilView( g_pCubeMapDepth, &DescDS, &(g_pCubeMapDepthViewArray[i]) ) ) )
        {
            DXUTTRACE( L"Failed to create depth stencil view for a depth cube map\n" );
            return false;
        }
    }

    // Create the shader resource view for the shadow map

    D3D10_SHADER_RESOURCE_VIEW_DESC SRVDesc;

    ZeroMemory( &SRVDesc, sizeof( SRVDesc ) );

    SRVDesc.Format = DXGI_FORMAT_R24_UNORM_X8_TYPELESS;
    SRVDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURECUBE;
    SRVDesc.Texture2DArray.MipLevels = 1;
    SRVDesc.Texture2DArray.MostDetailedMip = 0;
    SRVDesc.Texture2DArray.FirstArraySlice = 0;
    SRVDesc.Texture2DArray.ArraySize = 6;

    if( FAILED( pd3dDevice->CreateShaderResourceView( g_pCubeMapDepth, &SRVDesc, &g_pCubeMapTextureRV ) ) )
    {
        DXUTTRACE( L"Failed to create shader resource view for a depth stencil\n" );
        return false;
    }
    
    return true;
}

// Set matrices for cube mapping

void InitCubeMatrices( float * cubeCenter )
{
    D3DXVECTOR3 vLookDir;
    D3DXVECTOR3 vUpDir;
    D3DXMATRIX cubeViewMatrices[6];
    D3DXMATRIX cubeProjMatrix;

    vLookDir = D3DXVECTOR3( 1.0f, 0.0f, 0.0f ) + (* (D3DXVECTOR3 *) cubeCenter);
    vUpDir = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );
    D3DXMatrixLookAtLH( &cubeViewMatrices[0], (D3DXVECTOR3 *) cubeCenter, &vLookDir, &vUpDir );

    vLookDir = D3DXVECTOR3( -1.0f, 0.0f, 0.0f ) + (* (D3DXVECTOR3 *) cubeCenter);
    vUpDir = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );
    D3DXMatrixLookAtLH( &cubeViewMatrices[1], (D3DXVECTOR3 *) cubeCenter, &vLookDir, &vUpDir );

    vLookDir = D3DXVECTOR3( 0.0f, 1.0f,  0.0f ) + (* (D3DXVECTOR3 *) cubeCenter);
    vUpDir = D3DXVECTOR3( 0.0f, 0.0f, -1.0f );
    D3DXMatrixLookAtLH( &cubeViewMatrices[2], (D3DXVECTOR3 *) cubeCenter, &vLookDir, &vUpDir );

    vLookDir = D3DXVECTOR3( 0.0f, -1.0f, 0.0f ) + (* (D3DXVECTOR3 *) cubeCenter);
    vUpDir = D3DXVECTOR3( 0.0f,  0.0f, 1.0f );
    D3DXMatrixLookAtLH( &cubeViewMatrices[3], (D3DXVECTOR3 *) cubeCenter, &vLookDir, &vUpDir );

    vLookDir = D3DXVECTOR3( 0.0f, 0.0f, 1.0f ) + (* (D3DXVECTOR3 *) cubeCenter);
    vUpDir = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );
    D3DXMatrixLookAtLH( &cubeViewMatrices[4], (D3DXVECTOR3 *) cubeCenter, &vLookDir, &vUpDir );

    vLookDir = D3DXVECTOR3( 0.0f, 0.0f, -1.0f ) + (* (D3DXVECTOR3 *) cubeCenter);
    vUpDir = D3DXVECTOR3( 0.0f, 1.0f,  0.0f );
    D3DXMatrixLookAtLH( &cubeViewMatrices[5], (D3DXVECTOR3 *) cubeCenter, &vLookDir, &vUpDir );

    D3DXMatrixPerspectiveFovLH( &cubeProjMatrix, (float)D3DX_PI * 0.5f, 1.0f, 0.2f, 200.0f );

    g_pmCubeViewMatrixVariable->SetMatrixArray( (float *)cubeViewMatrices, 0, 6 );
    g_pmCubeProjMatrixVariable->SetMatrix( cubeProjMatrix );
}

void CALLBACK OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl * pControl, void * pUserContext )
{
    switch ( nControlID )
    {
    case IDC_TOGGLEFULLSCREEN: PostMessageW(DXUTGetHWND(), WM_SYSKEYDOWN, VK_RETURN, (1 << 29)); break;    
    case IDC_TOGGLEREF:        DXUTToggleREF(); break;
    case IDC_CHANGEDEVICE:     g_SettingsDlg.SetActive( !g_SettingsDlg.IsActive() ); break;
    case IDC_RESET:
        {
            g_bJitter = DEFAULT_JITTER;
            g_fSpeed = DEFAULT_SPEED;
            g_fNoiseScale = DEFAULT_NOISE_SCALE;
            g_fRoughness = DEFAULT_ROUGHNESS;
            g_fShapeSize = DEFAULT_SHAPE_SIZE;
            g_fFrequencyWeights [0] = DEFAULT_FREQUENCY1;
            g_fFrequencyWeights [1] = DEFAULT_FREQUENCY2;
            g_fFrequencyWeights [2] = DEFAULT_FREQUENCY3;
            g_fFrequencyWeights [3] = DEFAULT_FREQUENCY4;
            g_fFrequencyWeights [4] = DEFAULT_FREQUENCY5;
            
            g_SliderBar.GetSlider( IDC_NOISESCALE )->SetValue( (int) (g_fNoiseScale * 50) );
            g_SliderBar.GetSlider( IDC_ROUGHNESS )->SetValue( (int) (g_fRoughness * 25) );
            g_SliderBar.GetSlider( IDC_SHAPESIZE )->SetValue( (int) (g_fShapeSize * 25) );
            g_SliderBar.GetSlider( IDC_FREQUENCY1 )->SetValue( (int) (g_fFrequencyWeights[0] * 100) );
            g_SliderBar.GetSlider( IDC_FREQUENCY2 )->SetValue( (int) (g_fFrequencyWeights[1] * 100) );
            g_SliderBar.GetSlider( IDC_FREQUENCY3 )->SetValue( (int) (g_fFrequencyWeights[2] * 100) );
            g_SliderBar.GetSlider( IDC_FREQUENCY4 )->SetValue( (int) (g_fFrequencyWeights[3] * 100) );
            g_SliderBar.GetSlider( IDC_FREQUENCY5 )->SetValue( (int) (g_fFrequencyWeights[4] * 100) );
            
            WCHAR sz[200];
            StringCchPrintf( sz, 200, L"Flame size: %.2f", g_fShapeSize );
            g_SliderBar.GetStatic( IDC_SHAPESIZE_STATIC )->SetText( sz );
            StringCchPrintf( sz, 200, L"Base frequency: %.2f", g_fNoiseScale );
            g_SliderBar.GetStatic( IDC_NOISESCALE_STATIC )->SetText( sz );
            StringCchPrintf( sz, 200, L"Flame roughness: %.2f", g_fRoughness );
            g_SliderBar.GetStatic( IDC_ROUGHNESS_STATIC )->SetText( sz );
        }
        break;

    case IDC_FIRETECHNIQUE:
        {
            CDXUTComboBox* pComboBox = (CDXUTComboBox*) pControl;
            g_pCurrentTechnique = (ID3D10EffectTechnique*) pComboBox->GetSelectedData();
        }
        break;

    case IDC_FREQUENCY1:
    case IDC_FREQUENCY2:
    case IDC_FREQUENCY3:
    case IDC_FREQUENCY4:
    case IDC_FREQUENCY5:
        {
            CDXUTSlider * pSlider = (CDXUTSlider*) pControl;
            g_fFrequencyWeights[nControlID - IDC_FREQUENCY1] = pSlider->GetValue() / 100.0f;
        }
        break;

    case IDC_NOISESCALE:
        {
            CDXUTSlider * pSlider = (CDXUTSlider*) pControl;
            g_fNoiseScale = pSlider->GetValue() / 50.0f;

            WCHAR sz[200];
            StringCchPrintf( sz, 200, L"Base frequency: %.2f", g_fNoiseScale );

            g_SliderBar.GetStatic( IDC_NOISESCALE_STATIC )->SetText( sz );
        }
        break;

    case IDC_ROUGHNESS:
        {
            CDXUTSlider * pSlider = (CDXUTSlider*) pControl;
            g_fRoughness = pSlider->GetValue() / 25.0f;

            WCHAR sz[200];
            StringCchPrintf( sz, 200, L"Flame roughness: %.2f", g_fRoughness );

            g_SliderBar.GetStatic( IDC_ROUGHNESS_STATIC )->SetText( sz );
        }
        break;

    case IDC_SHAPESIZE:
        {
            CDXUTSlider * pSlider = (CDXUTSlider*) pControl;
            g_fShapeSize = pSlider->GetValue() / 25.0f;

            WCHAR sz[200];
            StringCchPrintf( sz, 200, L"Flame size: %.2f", g_fShapeSize );

            g_SliderBar.GetStatic( IDC_SHAPESIZE_STATIC )->SetText( sz );
        }
        break;

    case IDC_USEJITTERING:
        {
            CDXUTCheckBox * pCheckBox = (CDXUTCheckBox*) pControl;
            g_bJitter = pCheckBox -> GetChecked();
        }
        break;

    case IDC_SAMPLINGRATE:
        {
            CDXUTSlider * pSlider = (CDXUTSlider*) pControl;
            g_nSamplingRate = pSlider->GetValue();

            WCHAR sz[100];
            StringCchPrintf( sz, 100, L"Sampling Rate: %d", g_nSamplingRate ); 

            g_SampleUI.GetStatic( IDC_SAMPLINGRATE_STATIC )->SetText( sz );
        }
    }
}

// Initialize the app
void InitApp()
{
    g_SettingsDlg.Init( &g_DialogResourceManager );
    g_HUD.Init( &g_DialogResourceManager );
    g_SampleUI.Init( &g_DialogResourceManager );

    g_HUD.SetCallback( OnGUIEvent ); int iY = 10;
    g_HUD.AddButton( IDC_TOGGLEFULLSCREEN, L"Toggle full screen", 35, iY, 125, 22 );
    g_HUD.AddButton( IDC_TOGGLEREF, L"Toggle REF (F3)", 35, iY += 24, 125, 22, VK_F3 );
    g_HUD.AddButton( IDC_CHANGEDEVICE, L"Change device (F2)", 35, iY += 24, 125, 22, VK_F2 );
    g_HUD.AddButton( IDC_RESET, L"Reset", 35, iY += 24, 125, 22 );

    g_SampleUI.SetCallback( OnGUIEvent );

    iY = 0;
    g_SampleUI.AddCheckBox (IDC_USEJITTERING, L"Jittered sampling", 15, iY += 24, 125, 22, g_bJitter );

    iY += 24;

    WCHAR sz[200];
    StringCchPrintf( sz, 200, L"Sampling rate: %d", g_nSamplingRate ); 
    g_SampleUI.AddStatic( IDC_SAMPLINGRATE_STATIC, sz, 15, iY += 24, 125, 22 );

    g_SampleUI.AddSlider( IDC_SAMPLINGRATE, 0, iY += 24, 150, 22, MIN_SAMPLING_RATE, MAX_SAMPLING_RATE, g_nSamplingRate );

    CDXUTComboBox* pComboBox = NULL;
    g_SampleUI.AddComboBox( IDC_FIRETECHNIQUE, 0, iY += 48, 160, 24, 'S', false, &pComboBox );
    if( pComboBox )
        pComboBox->SetDropHeight( 30 );


    // Init slider bar

    g_SliderBar.Init( &g_DialogResourceManager );
    g_SliderBar.SetCallback( OnGUIEvent );

    iY = 10;
    StringCchPrintf( sz, 200, L"Base frequency: %.2f", g_fNoiseScale ); 
    g_SliderBar.AddStatic( IDC_NOISESCALE_STATIC, sz, 10, iY, 150, 22 );
    g_SliderBar.AddSlider( IDC_NOISESCALE, 10, iY += 24, 150, 22, 0, 100, (int) (g_fNoiseScale * 50) );
    StringCchPrintf( sz, 200, L"Flame roughness: %.2f", g_fRoughness ); 
    g_SliderBar.AddStatic( IDC_ROUGHNESS_STATIC, sz, 10, iY += 24, 150, 22);
    g_SliderBar.AddSlider( IDC_ROUGHNESS, 10, iY += 24, 150, 22, 0, 100, (int) (g_fRoughness * 25) );
    StringCchPrintf( sz, 200, L"Flame size: %.2f", g_fShapeSize ); 
    g_SliderBar.AddStatic( IDC_SHAPESIZE_STATIC, sz, 10, iY += 24, 150, 22);
    g_SliderBar.AddSlider( IDC_SHAPESIZE, 10, iY += 24, 150, 22, 25, 100, (int) (g_fShapeSize * 25) );

    iY += 24;
    //StringCchPrintf( sz, 200, L"Octaves: (%.4f, %.4f, %.4f, %.4f)", g_fFrequencyWeights[0], g_fFrequencyWeights[1], g_fFrequencyWeights[2], g_fFrequencyWeights[3], g_fFrequencyWeights[4]); 
    g_SliderBar.AddStatic( IDC_FREQUENCIES_STATIC, L"Octave weights", 10, iY += 24, 100, 22 );
    g_SliderBar.AddSlider( IDC_FREQUENCY1, 10, iY += 24, 150, 22, 0, 100, (int) (g_fFrequencyWeights[0] * 100) );
    g_SliderBar.AddSlider( IDC_FREQUENCY2, 10, iY += 24, 150, 22, 0, 100, (int) (g_fFrequencyWeights[1] * 100) );
    g_SliderBar.AddSlider( IDC_FREQUENCY3, 10, iY += 24, 150, 22, 0, 100, (int) (g_fFrequencyWeights[2] * 100) );
    g_SliderBar.AddSlider( IDC_FREQUENCY4, 10, iY += 24, 150, 22, 0, 100, (int) (g_fFrequencyWeights[3] * 100) );
    g_SliderBar.AddSlider( IDC_FREQUENCY5, 10, iY += 24, 150, 22, 0, 100, (int) (g_fFrequencyWeights[4] * 100) );
}

// Render text
void RenderText ()
{
    g_pTextHelper->Begin ();
    g_pTextHelper->SetInsertionPos( 5, 5 );
    g_pTextHelper->SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 0.0f, 1.0f ) );
    g_pTextHelper->DrawTextLine( DXUTGetFrameStats(true) );
    g_pTextHelper->DrawTextLine( DXUTGetDeviceStats() );
    g_pTextHelper->End ();
}

//--------------------------------------------------------------------------------------
// Reject any D3D10 devices that aren't acceptable by returning false
//--------------------------------------------------------------------------------------
bool CALLBACK IsD3D10DeviceAcceptable( UINT Adapter, UINT Output, D3D10_DRIVER_TYPE DeviceType, DXGI_FORMAT BackBufferFormat, bool bWindowed, void* pUserContext )
{
    return true;
}

//--------------------------------------------------------------------------------------
// Called right before creating a D3D9 or D3D10 device, allowing the app to modify the device settings as needed
//--------------------------------------------------------------------------------------
bool CALLBACK ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings, void* pUserContext )
{
    // For the first device created if its a REF device, optionally display a warning dialog box
    static bool s_bFirstTime = true;
    if( s_bFirstTime )
    {
        s_bFirstTime = false;
        if( (DXUT_D3D9_DEVICE == pDeviceSettings->ver && pDeviceSettings->d3d9.DeviceType == D3DDEVTYPE_REF) ||
            (DXUT_D3D10_DEVICE == pDeviceSettings->ver && pDeviceSettings->d3d10.DriverType == D3D10_DRIVER_TYPE_REFERENCE) )
            DXUTDisplaySwitchingToREFWarning( pDeviceSettings->ver );
    }

    return true;
}

//--------------------------------------------------------------------------------------
// Create any D3D10 resources that aren't dependant on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10CreateDevice( ID3D10Device * pd3dDevice, const DXGI_SURFACE_DESC * pBackBufferSurfaceDesc, void * pUserContext )
{
    HRESULT hr;

    V_RETURN( g_DialogResourceManager.OnD3D10CreateDevice( pd3dDevice ) );
    V_RETURN( g_SettingsDlg.OnD3D10CreateDevice( pd3dDevice ) );
    V_RETURN( D3DX10CreateFont( pd3dDevice, 15, 0, FW_BOLD, 1, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, L"Arial", &g_pFont ) );
    V_RETURN( D3DX10CreateSprite( pd3dDevice, 512, & g_pSprite ) );

    g_pTextHelper = new CDXUTTextHelper (NULL, NULL, g_pFont, g_pSprite, 15);

    D3D10_STATE_BLOCK_MASK stateMask;
    ZeroMemory( &stateMask, sizeof(stateMask) );
    stateMask.RSRasterizerState = 0xff;
    stateMask.OMBlendState = 0xff;
    stateMask.OMDepthStencilState = 0xff;
    stateMask.OMRenderTargets = 0xff;
    
    V_RETURN( D3D10CreateStateBlock( pd3dDevice, &stateMask, &g_pCurrentState ) );

    // Load effect from file

    wchar_t str [MAX_PATH];
    if (FAILED ( hr = DXUTFindDXSDKMediaFileCch( str, MAX_PATH, L"PerlinFire.fx" ) ) ) {
        MessageBox (NULL, L"PerlinFire.fx cannot be found.", L"Error", MB_OK);
        return hr;
    }

    ID3D10Blob *pErrors;
    if (D3DX10CreateEffectFromFile (str, NULL, NULL, "fx_4_0", D3D10_SHADER_ENABLE_STRICTNESS, 0, pd3dDevice, NULL, NULL, & g_pEffect, &pErrors, &hr ) != S_OK )
    {
        MessageBoxA (NULL, (char *)pErrors->GetBufferPointer(), "Compilation errors", MB_OK);
        return hr;
    }

    // Obtain techniques

    g_pPerlinFire3D = g_pEffect->GetTechniqueByName( "PerlinFire3D" );
    g_pPerlinFire3DFlow = g_pEffect->GetTechniqueByName( "PerlinFire3DFlow" );
    g_pPerlinFire4D = g_pEffect->GetTechniqueByName( "PerlinFire4D" );
    g_pGeometryTechnique = g_pEffect->GetTechniqueByName( "RenderGeometry" );
    g_pGeometryTechniqueAux = g_pEffect->GetTechniqueByName( "RenderGeometryAux" );

    // Populate combo-box
    CDXUTComboBox * pComboBox = g_SampleUI.GetComboBox( IDC_FIRETECHNIQUE );
    pComboBox->AddItem( L"4D simplex noise", g_pPerlinFire4D );
    pComboBox->AddItem( L"3D flow noise", g_pPerlinFire3DFlow );
    pComboBox->AddItem( L"3D simplex noise", g_pPerlinFire3D );

    g_pCurrentTechnique = g_pPerlinFire4D;

    // Obtain texture variables

    g_ptxScreenDepth = g_pEffect->GetVariableByName( "ScreenDepth" )->AsShaderResource();
    g_ptxSceneTexture = g_pEffect->GetVariableByName( "SceneTexture" )->AsShaderResource();
    g_ptxFireTex = g_pEffect->GetVariableByName( "FireShape" )->AsShaderResource();
    g_ptxJitterTex = g_pEffect->GetVariableByName( "JitterTexture" )->AsShaderResource();
    g_ptxPermTex = g_pEffect->GetVariableByName( "PermTexture" )->AsShaderResource();
    g_pCubeMapTextureVariable = g_pEffect->GetVariableByName( "ShadowMap" )->AsShaderResource();
    
    // Obtain miscellaneous variables

    g_pmCubeViewMatrixVariable = g_pEffect->GetVariableByName( "CubeViewMatrices" )->AsMatrix();
    g_pmCubeProjMatrixVariable = g_pEffect->GetVariableByName( "CubeProjectionMatrix" )->AsMatrix();
    g_pmWorldViewProj = g_pEffect->GetVariableByName( "WorldViewProj" )->AsMatrix();
    g_pvEyePos = g_pEffect->GetVariableByName( "EyePos" )->AsVector();
    g_pvLightPos = g_pEffect->GetVariableByName( "LightPos" )->AsVector();
    g_pfLightIntensity = g_pEffect->GetVariableByName( "LightIntensity" )->AsScalar();
    g_pfStepSize = g_pEffect->GetVariableByName( "StepSize" )->AsScalar();
    g_pfTime = g_pEffect->GetVariableByName( "Time" )->AsScalar();
    g_pfNoiseScale = g_pEffect->GetVariableByName( "NoiseScale" )->AsScalar();
    g_pfRoughness = g_pEffect->GetVariableByName( "Roughness" )->AsScalar();
    g_pfFrequencyWeights = g_pEffect->GetVariableByName( "FrequencyWeights" )->AsScalar();
    g_pbJitter = g_pEffect->GetVariableByName( "bJitter" )->AsScalar();
    g_piCubeMapFace = g_pEffect->GetVariableByName( "CubeMapFace" )->AsScalar();

    // Set input layouts

    D3D10_PASS_DESC PassDesc;

    D3D10_INPUT_ELEMENT_DESC geometryLayout [] = {
        { "POSITION", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 },
        { "NORMAL", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 12, D3D10_INPUT_PER_VERTEX_DATA, 0 },
        { "TEXCOORD", 0, DXGI_FORMAT_R32G32_FLOAT, 0, 24, D3D10_INPUT_PER_VERTEX_DATA, 0 }
    };

    unsigned int numElements = sizeof (geometryLayout) / sizeof (geometryLayout [0]);

    g_pGeometryTechnique->GetPassByIndex(0)->GetDesc( & PassDesc );
    V( pd3dDevice->CreateInputLayout( geometryLayout, numElements, PassDesc.pIAInputSignature,
        PassDesc.IAInputSignatureSize, & g_pGeometryVertexLayout ) )
    
    if (FAILED( hr = g_OutsideWorldMesh.Create(pd3dDevice, L"\\Orcs\\bonfire_wOrcs.x", geometryLayout, 3 ) ) ) {
         DXUTTRACE (L"Error %d while loading a mesh with outside scene\n", HRESULT_CODE (hr));
         return hr;
    }

    V( DXUTCreateBox( pd3dDevice, 1.0f, 1.0f, 1.0f, &g_pBoxMesh ) )

    // Load textures
    LoadTexture2D( pd3dDevice, L"Firetex.dds", & g_pFireTexture, & g_pFireTextureSRV );

    // Create noise texture
    // Fill the texture with random numbers from 0 to 256
    srand( GetTickCount() );

    BYTE data[256 * 256];
    for (int i = 0; i < 256 * 256; i++)
    {
        data[i] = (unsigned char) (rand () % 256);
    }

    D3D10_TEXTURE2D_DESC desc;
    desc.Width = 256;
    desc.Height = 256;
    desc.MipLevels = 1;
    desc.ArraySize = 1;
    desc.Format = DXGI_FORMAT_R8_TYPELESS;
    desc.SampleDesc.Count = 1;
    desc.SampleDesc.Quality = 0;
    desc.Usage = D3D10_USAGE_IMMUTABLE;
    desc.BindFlags = D3D10_BIND_SHADER_RESOURCE;
    desc.CPUAccessFlags = 0;
    desc.MiscFlags = 0;

    D3D10_SUBRESOURCE_DATA dataDesc;
    dataDesc.pSysMem = data;
    dataDesc.SysMemPitch = 256;

    V( pd3dDevice->CreateTexture2D(&desc, &dataDesc, &g_pNoiseTexture) );

    // Create the shader resource view for jittering
    D3D10_SHADER_RESOURCE_VIEW_DESC descSRV;

    ZeroMemory( &descSRV, sizeof(descSRV) );
    descSRV.Format = DXGI_FORMAT_R8_UNORM;
    descSRV.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
    descSRV.Texture2D.MipLevels = 1;
    descSRV.Texture2D.MostDetailedMip = 0;

    V( pd3dDevice->CreateShaderResourceView( g_pNoiseTexture, &descSRV, &g_pJitterTextureSRV ) );

    // Create the shader resource view for permutation
    descSRV.Format = DXGI_FORMAT_R8_UINT;

    V( pd3dDevice->CreateShaderResourceView( g_pNoiseTexture, &descSRV, &g_pPermTextureSRV ) );

    g_ptxFireTex -> SetResource (g_pFireTextureSRV);
    g_ptxJitterTex -> SetResource (g_pJitterTextureSRV);
    g_ptxPermTex -> SetResource (g_pPermTextureSRV);

    // Setup camera
    D3DXVECTOR3 vEye(0.0f, 15.0f, -30.0f);
    D3DXVECTOR3 vAt(0, 7.0f, 0);

    g_Camera.SetViewParams( &vEye, &vAt );
    g_Camera.SetModelCenter( D3DXVECTOR3(0, 7.0f, 0 ) );
    g_Camera.SetButtonMasks( 0, MOUSE_WHEEL, MOUSE_LEFT_BUTTON );
    
    // Setup cubemap
    
    PrepareCubeMap( pd3dDevice );
    
    g_pCubeMapTextureVariable->SetResource( g_pCubeMapTextureRV );
    
    return S_OK;
}

//--------------------------------------------------------------------------------------
// Create any D3D10 resources that depend on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10ResizedSwapChain( ID3D10Device* pd3dDevice, IDXGISwapChain *pSwapChain, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
    HRESULT hr;

    V_RETURN( g_DialogResourceManager.OnD3D10ResizedSwapChain( pd3dDevice, pBackBufferSurfaceDesc ) );
    V_RETURN( g_SettingsDlg.OnD3D10ResizedSwapChain( pd3dDevice, pBackBufferSurfaceDesc ) );

    g_HUD.SetLocation( pBackBufferSurfaceDesc->Width - 170, 0);
    g_HUD.SetSize( 170, 170 );
    g_SampleUI.SetLocation( pBackBufferSurfaceDesc->Width-170, pBackBufferSurfaceDesc->Height-300 );
    g_SampleUI.SetSize( 170, 300 );

    g_SliderBar.SetLocation (10, pBackBufferSurfaceDesc->Height - 350);

    // Setup the camera's projection parameters
    float aspectRatio = pBackBufferSurfaceDesc->Width / (float)pBackBufferSurfaceDesc->Height;
    g_Camera.SetProjParams(D3DX_PI / 3, aspectRatio, 1.0f, 1000.0f);
    g_Camera.SetWindow( pBackBufferSurfaceDesc->Width, pBackBufferSurfaceDesc->Height );

    D3D10_TEXTURE2D_DESC descDepth;
    descDepth.Width = pBackBufferSurfaceDesc->Width;
    descDepth.Height = pBackBufferSurfaceDesc->Height;
    descDepth.MipLevels = 1;
    descDepth.ArraySize = 1;
    descDepth.Format = DXGI_FORMAT_R32_TYPELESS;
    descDepth.SampleDesc.Count = 1;
    descDepth.SampleDesc.Quality = 0;
    descDepth.Usage = D3D10_USAGE_DEFAULT;
    descDepth.BindFlags = D3D10_BIND_DEPTH_STENCIL | D3D10_BIND_SHADER_RESOURCE;
    descDepth.CPUAccessFlags = 0;
    descDepth.MiscFlags = 0;
    V( pd3dDevice->CreateTexture2D( &descDepth, NULL, &g_pDepthBuffer ));

    // Create the depth stencil view
    D3D10_DEPTH_STENCIL_VIEW_DESC descDSV;
    descDSV.Format = DXGI_FORMAT_D32_FLOAT;
    descDSV.ViewDimension = D3D10_DSV_DIMENSION_TEXTURE2D;
    descDSV.Texture2D.MipSlice = 0;
    V( pd3dDevice->CreateDepthStencilView( g_pDepthBuffer, &descDSV, &g_pDepthBufferDSV ) );

    // Create the shader resource view
    D3D10_SHADER_RESOURCE_VIEW_DESC descSRV;
    descSRV.Format = DXGI_FORMAT_R32_FLOAT;
    descSRV.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
    descSRV.Texture2D.MipLevels            = 1;
    descSRV.Texture2D.MostDetailedMip      = 0;
    V( pd3dDevice -> CreateShaderResourceView (g_pDepthBuffer, &descSRV, &g_pDepthBufferSRV ) );

    g_ptxScreenDepth->SetResource( g_pDepthBufferSRV );

    return S_OK;
}

//--------------------------------------------------------------------------------------
// Handle updates to the scene.  This is called regardless of which D3D API is used
//--------------------------------------------------------------------------------------
void CALLBACK OnFrameMove( double fTime, float fElapsedTime, void* pUserContext )
{
    g_Camera.FrameMove( fElapsedTime );
}

//--------------------------------------------------------------------------------------
// Render the scene using the D3D10 device
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10FrameRender( ID3D10Device* pd3dDevice, double fTime, float fElapsedTime, void* pUserContext )
{
    ID3D10RenderTargetView * pBackBufferRTV = DXUTGetD3D10RenderTargetView();
    ID3D10DepthStencilView * pBackBufferDSV = DXUTGetD3D10DepthStencilView();

    // Clear render target and the depth stencil 
    float ClearColor[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
    pd3dDevice->ClearRenderTargetView( pBackBufferRTV, ClearColor );
    pd3dDevice->ClearDepthStencilView( pBackBufferDSV, D3D10_CLEAR_DEPTH, 1.0, 0 );

    // Set matrices
    D3DXMATRIX mWorld = *g_Camera.GetWorldMatrix();
    D3DXMATRIX mView = *g_Camera.GetViewMatrix();
    D3DXMATRIX mProj = *g_Camera.GetProjMatrix();

    D3DXMATRIX mViewProj = mView * mProj;
    D3DXMATRIX mWorldView = mWorld * mView;
    D3DXMATRIX mWorldViewProj = mWorldView * mProj;

    g_pmWorldViewProj -> SetMatrix ((float *) & mWorldViewProj );
    
    float rnd = (float)rand() / RAND_MAX * 0.5f + 0.5f;
    float lightPos[] = { 0.25f * (rnd - 0.5f), 5.7f, 1.0f * (rnd - 0.5f), 1.0f };
    
    InitCubeMatrices( lightPos );
    
    g_pvLightPos->SetFloatVector( lightPos );

    // First, render an additional z-buffer for reads in the shader
    // Unfortunately DX10 doesn't allow to set a currently bound depth buffer
    // as a shader resource even if z-writes are explicitly disabled

    pd3dDevice->ClearDepthStencilView( g_pDepthBufferDSV, D3D10_CLEAR_DEPTH, 1.0, 0 );

    ID3D10RenderTargetView * pNullView [] = { NULL };
    pd3dDevice->OMSetRenderTargets( 1, pNullView, g_pDepthBufferDSV );

    g_pGeometryTechnique->GetPassByIndex(0)->Apply(0);
    pd3dDevice->IASetInputLayout( g_pGeometryVertexLayout );
    g_OutsideWorldMesh.Render( pd3dDevice );
    
    // Second, render the whole scene to a shadowmap
    // Set a new viewport for rendering to cube map

    D3D10_VIEWPORT SMVP, prevSMVP;
    unsigned int prevViewpotrNum = 1;
    
    SMVP.Height		= g_CubeMapSize;
    SMVP.Width		= g_CubeMapSize;
    SMVP.MinDepth	= 0;
    SMVP.MaxDepth	= 1;
    SMVP.TopLeftX	= 0;
    SMVP.TopLeftY	= 0;

    pd3dDevice->RSGetViewports( &prevViewpotrNum, &prevSMVP );
    pd3dDevice->RSSetViewports( 1, &SMVP );
    
    pd3dDevice->IASetInputLayout( g_pGeometryVertexLayout );
    for( int i = 0; i < 6; i ++ )
    {
        g_piCubeMapFace->SetInt(i);

        pd3dDevice->ClearDepthStencilView( g_pCubeMapDepthViewArray[i], D3D10_CLEAR_DEPTH, 1.0f, 0 );
        pd3dDevice->OMSetRenderTargets( 1, pNullView, g_pCubeMapDepthViewArray[i] );
        
        g_pGeometryTechniqueAux->GetPassByIndex(0)->Apply(0);
        g_OutsideWorldMesh.Render( pd3dDevice );
    }

    // Switch back to main RTs

    pd3dDevice->RSSetViewports( 1, &prevSMVP );

    pd3dDevice->OMSetRenderTargets( 1, &pBackBufferRTV, pBackBufferDSV );

    // If the settings dialog is being shown, then render it instead of rendering the app's scene
    if( g_SettingsDlg.IsActive() )
    {
        g_SettingsDlg.OnRender( fElapsedTime );
        return;
    }

    g_pfNoiseScale->SetFloat( g_fNoiseScale );
    g_pfRoughness->SetFloat( g_fRoughness );
    g_pfFrequencyWeights->SetFloatArray( g_fFrequencyWeights, 0, 5 );

    g_pfTime->SetFloat( (float)fTime * g_fSpeed );
    g_pfStepSize->SetFloat( (float)1.0f / g_nSamplingRate );
    g_pbJitter->SetBool( g_bJitter );
    
    g_pGeometryTechnique->GetPassByIndex(1)->Apply(0);
    pd3dDevice->IASetInputLayout( g_pGeometryVertexLayout );
    g_OutsideWorldMesh.Render( pd3dDevice );
    
    // Render fire volume

    D3DXMATRIX mTranslate, mScale, mWorldViewInv;
    D3DXMatrixTranslation( &mTranslate, 0, 0.5f, 0 );
    D3DXMatrixScaling( &mScale, 4.0f * g_fShapeSize, 8.0f * g_fShapeSize, 4.0f * g_fShapeSize);

    mWorldView = mTranslate * mScale * mWorld * mView;
    mWorldViewProj = mWorldView * mProj;

    D3DXMatrixInverse( &mWorldViewInv, NULL, &mWorldView );

    D3DXVECTOR4 vEye;
    D3DXVECTOR4 v(0, 0, 0, 1);
    D3DXVec4Transform( &vEye, &v, &mWorldViewInv );

    g_pmWorldViewProj->SetMatrix ( (float *) & mWorldViewProj );
    g_pvEyePos->SetFloatVector ( (float *) &vEye );
    g_pfLightIntensity->SetFloat( rnd );

    g_pCurrentTechnique->GetPassByIndex(0)->Apply(0);
    g_pBoxMesh->DrawSubset( 0 );

    RenderText();

    // Render GUI

    DXUT_BeginPerfEvent( DXUT_PERFEVENTCOLOR, L"HUD / Stats" );
    RenderText();
    g_HUD.OnRender( fElapsedTime ); 
    g_SampleUI.OnRender( fElapsedTime );
    g_SliderBar.OnRender( fElapsedTime );
    DXUT_EndPerfEvent();
}

//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10ResizedSwapChain 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10ReleasingSwapChain( void* pUserContext )
{
    g_DialogResourceManager.OnD3D10ReleasingSwapChain();

    ID3D10RenderTargetView * pNullView = { NULL };
    DXUTGetD3D10Device()->OMSetRenderTargets( 1, &pNullView, NULL );

    SAFE_RELEASE (g_pDepthBufferDSV);
    SAFE_RELEASE (g_pDepthBufferSRV);
    SAFE_RELEASE (g_pDepthBuffer);
}

//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10CreateDevice 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10DestroyDevice( void* pUserContext )
{
    g_DialogResourceManager.OnD3D10DestroyDevice();
    g_SettingsDlg.OnD3D10DestroyDevice();

    SAFE_RELEASE( g_pPermTextureSRV );
    SAFE_RELEASE( g_pJitterTextureSRV );
    SAFE_RELEASE( g_pNoiseTexture );
    SAFE_RELEASE( g_pFireTextureSRV );
    SAFE_RELEASE( g_pFireTexture );
    
    SAFE_RELEASE( g_pCubeMapDepth );
    SAFE_RELEASE( g_pCubeMapTextureRV );

    for( int i = 0; i < 6; i ++ )
    {
        SAFE_RELEASE( g_pCubeMapDepthViewArray[i] );
    }

    SAFE_RELEASE( g_pBoxMesh );

    g_OutsideWorldMesh.Destroy();

    SAFE_RELEASE( g_pGeometryVertexLayout );

    SAFE_RELEASE( g_pEffect );

    SAFE_RELEASE( g_pCurrentState );

    SAFE_DELETE( g_pTextHelper );
    SAFE_RELEASE( g_pSprite );
    SAFE_RELEASE( g_pFont );
}

//--------------------------------------------------------------------------------------
// Handle messages to the application
//--------------------------------------------------------------------------------------
LRESULT CALLBACK MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, 
                         bool* pbNoFurtherProcessing, void* pUserContext )
{
    // Pass messages to dialog resource manager calls so GUI state is updated correctly
    *pbNoFurtherProcessing = g_DialogResourceManager.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing )
        return 0;

    // Pass messages to settings dialog if its active
    if( g_SettingsDlg.IsActive() )
    {
        g_SettingsDlg.MsgProc( hWnd, uMsg, wParam, lParam );
        return 0;
    }

    // Give the dialogs a chance to handle the message first
    *pbNoFurtherProcessing = g_HUD.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing )
        return 0;

    *pbNoFurtherProcessing = g_SampleUI.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing )
        return 0;

    *pbNoFurtherProcessing = g_SliderBar.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing )
        return 0;

    // Pass all remaining windows messages to camera so it can respond to user input
    g_Camera.HandleMessages( hWnd, uMsg, wParam, lParam );

    return 0;
}

//--------------------------------------------------------------------------------------
// Handle key presses
//--------------------------------------------------------------------------------------
void CALLBACK OnKeyboard( UINT nChar, bool bKeyDown, bool bAltDown, void* pUserContext )
{
}

//--------------------------------------------------------------------------------------
// Handle mouse button presses
//--------------------------------------------------------------------------------------
void CALLBACK OnMouse( bool bLeftButtonDown, bool bRightButtonDown, bool bMiddleButtonDown, 
                      bool bSideButton1Down, bool bSideButton2Down, int nMouseWheelDelta, 
                      int xPos, int yPos, void* pUserContext )
{
}


//--------------------------------------------------------------------------------------
// Call if device was removed.  Return true to find a new device, false to quit
//--------------------------------------------------------------------------------------
bool CALLBACK OnDeviceRemoved( void* pUserContext )
{
    return true;
}

//--------------------------------------------------------------------------------------
// Initialize everything and go into a render loop
//--------------------------------------------------------------------------------------
int WINAPI wWinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nCmdShow )
{
    // Enable run-time memory check for debug builds.
#if defined(DEBUG) | defined(_DEBUG)
    _CrtSetDbgFlag( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
#endif

    // Set general DXUT callbacks
    DXUTSetCallbackFrameMove( OnFrameMove );
    DXUTSetCallbackKeyboard( OnKeyboard );
    DXUTSetCallbackMouse( OnMouse );
    DXUTSetCallbackMsgProc( MsgProc );
    //DXUTSetCallbackDeviceChanging( ModifyDeviceSettings );
    DXUTSetCallbackDeviceRemoved( OnDeviceRemoved );

    // Set the D3D10 DXUT callbacks. Remove these sets if the app doesn't need to support D3D10
    DXUTSetCallbackD3D10DeviceAcceptable( IsD3D10DeviceAcceptable );
    DXUTSetCallbackD3D10DeviceCreated( OnD3D10CreateDevice );
    DXUTSetCallbackD3D10SwapChainResized( OnD3D10ResizedSwapChain );
    DXUTSetCallbackD3D10FrameRender( OnD3D10FrameRender );
    DXUTSetCallbackD3D10SwapChainReleasing( OnD3D10ReleasingSwapChain );
    DXUTSetCallbackD3D10DeviceDestroyed( OnD3D10DestroyDevice );

    // IMPORTANT: set SDK media search path to include source directory of this sample, when started from .\Bin
    HRESULT hr;
    V_RETURN(DXUTSetMediaSearchPath(L"..\\Source\\PerlinFire"));
    
    InitApp();

    DXUTInit( true, true, NULL ); // Parse the command line, show msgboxes on error, no extra command line params
    DXUTSetCursorSettings( true, true ); // Show the cursor and clip it when in full screen
    DXUTCreateWindow( L"Perlin Fire" );
    DXUTCreateDevice( true, 640, 480 );  
    DXUTMainLoop(); // Enter into the DXUT render loop

    // Perform any application-level cleanup here

    return DXUTGetExitCode();
}
